part of mahg_essential_package;

abstract class MahgModel {
  /// return the id valie
  String get idValue;

  /// return display name
  String get displayName;

  /// return bool from dynamic
  bool getBool(dynamic val) => MahgModel.getBoolEx(val);

  /// return bool from dynamic
  static bool getBoolEx(dynamic val) {
    if (val == null) return false;
    if (val is bool) return val;
    if (val is String) return val == 'true' || val == '1';
    if (val is num) return val == 1;
    return false;
  }

  /// return DateTime from dynamic
  DateTime? getDate(dynamic val) => MahgModel.getDateEx(val);

  /// return DateTime from dynamic
  static DateTime? getDateEx(dynamic val) {
    if (val == null) return null;
    if (val is DateTime) return val;
    if (val is String) return parseUTC(val);
    return null;
  }

  /// return double from dynamic
  double? getDouble(dynamic val, [bool defaultIsZero = true]) =>
      MahgModel.getDoubleEx(val, defaultIsZero);

  /// return double from dynamic
  static double? getDoubleEx(dynamic val, [bool defaultIsZero = true]) {
    if (val == null) return defaultIsZero ? 0 : null;
    if (val is num) return val.toDouble();
    if (val is String)
      return double.tryParse(val) ?? (defaultIsZero ? 0 : null);
    return (defaultIsZero ? 0 : null);
  }

  /// return true if the Id is default
  static bool isDefault(String? id) => id != null && id.length < 30;

  /// convert service time to DateTime UTC
  static DateTime? parseUTC(String? serverValue,
      {String format = "yyyy-MM-ddTHH:mm", DateTime? defaultValue}) {
    if (serverValue == null) return defaultValue;
    try {
      return intl.Intl.withLocale(
          'en', () => intl.DateFormat(format).parseUTC(serverValue));
    } catch (err) {
      return defaultValue;
    }
  }
}
