part of mahg_essential_package;

class AppLocalizations {
  final Locale locale;

  AppLocalizations(this.locale);

  // Helper method to keep the code in the widgets concise
  // Localizations are accessed using an InheritedWidget "of" syntax
  static AppLocalizations? of(BuildContext context) {
    return Localizations.of<AppLocalizations>(context, AppLocalizations);
  }

// Static member to have a simple access to the delegate from the MaterialApp
  static const LocalizationsDelegate<AppLocalizations> delegate =
      _AppLocalizationsDelegate();

  Map<String, dynamic>? _sentences;
  static Map<String, String>? data;

  Future<bool> load() async {
    _sentences = locale.languageCode == 'ar' ? data : {};

    return true;
  }

  // This method will be called from every widget which needs a localized text
  String translate(String key) {
    if (key == null) return '';
    try {
      if (this._sentences == null) {
        print('_sentences is null');
        return key;
      }
      return (this._sentences?.containsKey(key) ?? false)
          ? (this._sentences?[key] ?? key)
          : key;
    } catch (ex) {
      print('trans error : $ex');
      return key;
    }
  }
}

// LocalizationsDelegate is a factory for a set of localized resources
// In this case, the localized strings will be gotten in an AppLocalizations object
class _AppLocalizationsDelegate
    extends LocalizationsDelegate<AppLocalizations> {
  // This delegate instance will never change (it doesn't even have fields!)
  // It can provide a constant constructor.
  const _AppLocalizationsDelegate();

  @override
  bool isSupported(Locale locale) {
    // Include all of your supported language codes here
    return true;
  }

  @override
  Future<AppLocalizations> load(Locale locale) async {
    // AppLocalizations class is where the JSON loading actually runs
    AppLocalizations localizations = new AppLocalizations(locale);
    await localizations.load();
    return localizations;
  }

  @override
  bool shouldReload(_AppLocalizationsDelegate old) => false;
}

///
///
///
/// SIMPLE EXAMPL FOR USING THIS
///
///
///

// void main() async {
//   WidgetsFlutterBinding.ensureInitialized();
//   await ConfigService.init();
//   initLocalization();
//   runApp(RestartWidget(MyApp()));
// }

// class MyApp extends StatelessWidget {
//   @override
//   Widget build(BuildContext context) {
//     return MaterialApp(
//       title: 'Flutter Demo',
//       theme: ThemeData(
//         primarySwatch: Colors.blue,
//       ),
//       // locale: Locale('ar', 'SA'),
//       // List all of the app's supported locales here
//       supportedLocales: [
//         Locale('en', 'US'),
//         Locale('ar', 'SA'),
//       ],
//       // These delegates make sure that the localization data for the proper language is loaded
//       localizationsDelegates: [
//         GlobalCupertinoLocalizations.delegate,
//         DefaultCupertinoLocalizations.delegate,
//         AppLocalizations.delegate,

//         // Built-in localization of basic text for Material widgets
//         GlobalMaterialLocalizations.delegate,
//         // Built-in localization for text direction LTR/RTL
//         GlobalWidgetsLocalizations.delegate,
//       ],
//       localeResolutionCallback: (locale, supportedLocales) {
//         print('Check Language');
//         print(locale?.languageCode);

//         var lang = ConfigService.getValueString('language');
//         if (ConfigService.getValueString('language') == '') {
//           for (Locale supportedLocale in supportedLocales) {
//             if (supportedLocale.languageCode == locale?.languageCode ||
//                 supportedLocale.countryCode == locale?.countryCode) {
//               ConfigService.setValueString(
//                   'language', supportedLocale.languageCode);
//               lang = supportedLocale.languageCode;
//               print('xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx${lang}');
//               return supportedLocale;
//             }
//           }
//         } else {
//           print(
//               'else xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx${ConfigService.getValueString('language')}');
//           var local = supportedLocales
//               .where((element) => element.languageCode == lang)
//               .first;
//           return local;
//         }
//       },

//       home: MyHomePage(),
//     );
//   }
// }

// class MyHomePage extends StatelessWidget {
//   @override
//   Widget build(BuildContext context) {
//     return Scaffold(
//       body: Center(
//         child: Padding(
//           padding: const EdgeInsets.all(8.0),
//           child: Column(
//             mainAxisSize: MainAxisSize.min,
//             children: <Widget>[
//               Text(
//                 AppLocalizations.of(context)!.translate('Login'),
//                 style: TextStyle(fontSize: 25),
//                 textAlign: TextAlign.center,
//               ),
//               SizedBox(height: 10),
//               Text(
//                 AppLocalizations.of(context)!.translate('Location'),
//                 style: TextStyle(fontSize: 25),
//                 textAlign: TextAlign.center,
//               ),
//               SizedBox(height: 10),
//               Text(
//                 AppLocalizations.of(context)!.translate('Phone Number'),
//                 style: TextStyle(fontSize: 25),
//                 textAlign: TextAlign.center,
//               ),
//               SizedBox(height: 10),
//               Text(
//                 'This will not be translated.',
//                 style: TextStyle(fontSize: 25),
//                 textAlign: TextAlign.center,
//               ),
//               SizedBox(height: 10),
//               ElevatedButton(
//                   onPressed: () async {
//                     var x =
//                         await ConfigService.setValueString('language', 'en');
//                     print('aaaaaaaaaaaaaaaaaa${x}');
//                     RestartWidget.restartApp(context);
//                   },
//                   child: Text('test'))
//             ],
//           ),
//         ),
//       ),
//     );
//   }
// }


///local_service.dart
///
///
///
///

// void initLocalization() {
//   AppLocalizations.data = {
//     'Login': 'تسجيل الدخول',
//     'Location': 'الموقع',
//     'Phone Number': 'رقم الهاتف'
//   };
// }
