part of mahg_essential_package;

class LoadingOverlay {
  static Future<LoadingDialogResult<T>> showFutureLoadingDialog<T>({
    required BuildContext context,
    required Future<T> Function() future,
    String? title,
    String? backLabel,
    String Function(dynamic exception)? onError,
    bool barrierDismissible = false,
    bool isCupertinoStyle = false,
    bool isArabic = false,
  }) async {
    final dialog = LoadingDialog<T>(
      future: future,
      title: title,
      backLabel: backLabel,
      onError: onError,
      isCupertinoStyle: isCupertinoStyle,
      isArabic: isArabic,
    );
    final result = dialog.isCupertinoStyle
        ? await showCupertinoDialog<LoadingDialogResult<T>>(
            barrierDismissible: barrierDismissible,
            context: context,
            builder: (BuildContext context) => dialog,
          )
        : await showDialog<LoadingDialogResult<T>>(
            context: context,
            barrierDismissible: barrierDismissible,
            builder: (BuildContext context) => dialog,
          );
    return result ??
        LoadingDialogResult<T>(
          error: Exception('FutureDialog canceled'),
          stackTrace: StackTrace.current,
        );
  }
}

class LoadingDialog<T> extends StatefulWidget {
  final String? title;
  final String? backLabel;
  final bool isCupertinoStyle;
  final bool isArabic;
  final Future<T> Function() future;
  final String Function(dynamic exception)? onError;

  static String defaultTitle = 'Loading... Please Wait!';
  static String defaultBackLabel = 'Back';
  // ignore: prefer_function_declarations_over_variables
  static String Function(dynamic exception) defaultOnError =
      (exception) => exception.toString();

  const LoadingDialog({
    Key? key,
    required this.future,
    this.title,
    this.onError,
    this.backLabel,
    this.isCupertinoStyle = false,
    this.isArabic = false,
  }) : super(key: key);
  @override
  _LoadingDialogState<T> createState() => _LoadingDialogState<T>();
}

class _LoadingDialogState<T> extends State<LoadingDialog> {
  dynamic exception;
  StackTrace? stackTrace;

  @override
  void initState() {
    super.initState();
    widget.future().then(
        (result) => Navigator.of(context)
            .pop<LoadingDialogResult<T>>(LoadingDialogResult(result: result)),
        onError: (e, s) => setState(() {
              exception = e;
              stackTrace = s;
            }));
  }

  @override
  Widget build(BuildContext context) {
    final titleLabel = exception != null
        ? widget.onError?.call(exception) ??
            LoadingDialog.defaultOnError(exception)
        : widget.title ??
            (widget.isArabic
                ? "جاري التحميل الرجاءالانتظار!"
                : LoadingDialog.defaultTitle);
    final content = Column(
      mainAxisSize: MainAxisSize.min,
      children: [
        Padding(
          padding: const EdgeInsets.all(8.0),
          child: exception == null
              ? const CircularProgressIndicator.adaptive()
              : const Icon(
                  Icons.error_outline_outlined,
                  color: Colors.red,
                ),
        ),
        const SizedBox(
          height: 15,
        ),
        Text(titleLabel),
        const SizedBox(
          height: 5,
        ),
      ],
    );

    if (widget.isCupertinoStyle) {
      return CupertinoAlertDialog(
        content: content,
        actions: [
          if (exception != null)
            CupertinoDialogAction(
              onPressed: Navigator.of(context).pop,
              child: Text(widget.backLabel ??
                  (widget.isArabic ? "رجوع" : LoadingDialog.defaultBackLabel)),
            )
        ],
      );
    }
    return AlertDialog(
      content: content,
      actions: [
        if (exception != null)
          TextButton(
            onPressed: () => Navigator.of(context).pop<LoadingDialogResult<T>>(
              LoadingDialogResult(
                error: exception,
                stackTrace: stackTrace,
              ),
            ),
            child: Text(widget.backLabel ??
                (widget.isArabic ? "رجوع" : LoadingDialog.defaultBackLabel)),
          ),
      ],
    );
  }
}

class LoadingDialogResult<T> {
  final T? result;
  final dynamic error;
  final StackTrace? stackTrace;

  LoadingDialogResult({this.result, this.error, this.stackTrace});
}
