part of mahg_essential_package;

abstract class MahgStatefulWidget<C extends MahgController>
    extends StatefulWidget {
  final C? controller;
  const MahgStatefulWidget(this.controller, {super.key});
}

abstract class MahgState<T extends MahgStatefulWidget, C extends MahgController>
    extends State<T> with LifecycleAware, LifecycleMixin {
  late C controller;

  /// Whether this [State] object is displayed on the screen.
  bool isDislplayed = false;
  Future<void> onDisplayed(Widget page, bool _isDislplayed) async {
    controller.onDisplayed(this.widget, _isDislplayed);
  }

  C createController();

  @override
  void onLifecycleEvent(LifecycleEvent event) {
    if (event == LifecycleEvent.active) {
      controller.state = this;
      isDislplayed = true;
    } else {
      isDislplayed = false;
    }
    onDisplayed(widget, isDislplayed);
  }

  @override
  initState() {
    controller = createController();
    controller.init(widget);
    controller.initLate(widget).then((value) => controller.setState());
    super.initState();
  }

  @override
  dispose() {
    controller.dispose();
    super.dispose();
  }
}
