part of mahg_essential_package;

class TextFieldWidget extends StatefulWidget {
  final TextEditingController controller;
  final String labelText;
  final String hintText;
  final String? regExp;
  final String? validateMessage;
  final TextDirection textDirection;
  final TextAlign textAlign;
  final bool isObscure;

  final TextInputType? textInputType;
  const TextFieldWidget(
      {required this.controller,
      required this.labelText,
      required this.hintText,
      required this.isObscure,
      this.textInputType,
      this.validateMessage,
      this.regExp,
      required this.textAlign,
      required this.textDirection,
      Key? key})
      : super(key: key);

  @override
  State<TextFieldWidget> createState() => _TextFieldWidgetState();
}

class _TextFieldWidgetState extends State<TextFieldWidget> {
  bool isshown = false;

  @override
  void initState() {
    isshown = this.widget.isObscure;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: Directionality(
        textDirection: TextDirection.rtl,
        child: TextFormField(
            obscureText: isshown,
            keyboardType: this.widget.textInputType,
            controller: widget.controller,
            textAlign: TextAlign.start,
            decoration: textInputDecoration(
                lableText: this.widget.labelText,
                hintText: this.widget.hintText),
            validator: (val) {
              if ((val!.isEmpty) ||
                  !RegExp(this.widget.regExp ?? '').hasMatch(val)) {
                return this.widget.validateMessage ?? null;
              }
            }),
      ),
      decoration: inputBoxDecorationShaddow(),
    );
  }

  InputDecoration textInputDecoration(
      {String lableText = "", String hintText = ""}) {
    return InputDecoration(
      labelText: lableText,
      hintText: hintText,
      fillColor: Colors.white,
      filled: true,
      suffixIcon: this.widget.isObscure
          ? IconButton(
              icon: Icon(isshown ? Icons.visibility : Icons.visibility_off),
              onPressed: () {
                setState(() {
                  isshown = !isshown;
                });
              },
            )
          : Padding(padding: EdgeInsets.all(0)),
      contentPadding: EdgeInsets.fromLTRB(20, 10, 20, 10),
      focusedBorder: OutlineInputBorder(
          borderRadius: BorderRadius.circular(100.0),
          borderSide: BorderSide(color: Colors.grey)),
      enabledBorder: OutlineInputBorder(
          borderRadius: BorderRadius.circular(100.0),
          borderSide: BorderSide(color: Colors.grey.shade400)),
      errorBorder: OutlineInputBorder(
          borderRadius: BorderRadius.circular(100.0),
          borderSide: BorderSide(color: Colors.red, width: 2.0)),
      focusedErrorBorder: OutlineInputBorder(
          borderRadius: BorderRadius.circular(100.0),
          borderSide: BorderSide(color: Colors.red, width: 2.0)),
    );
  }
}

InputDecoration textInputDecoration(
    {String lableText = "", String hintText = ""}) {
  bool _showPassword = false;
  void _togglevisibility() {
    _showPassword = !_showPassword;
  }

  return InputDecoration(
    labelText: lableText,
    hintText: hintText,
    fillColor: Colors.white,
    filled: true,
    suffixIcon: GestureDetector(
      onTap: () {
        _togglevisibility();
      },
      child: Icon(
        _showPassword ? Icons.visibility : Icons.visibility_off,
        color: Colors.red,
      ),
    ),
    contentPadding: EdgeInsets.fromLTRB(20, 10, 20, 10),
    focusedBorder: OutlineInputBorder(
        borderRadius: BorderRadius.circular(100.0),
        borderSide: BorderSide(color: Colors.grey)),
    enabledBorder: OutlineInputBorder(
        borderRadius: BorderRadius.circular(100.0),
        borderSide: BorderSide(color: Colors.grey.shade400)),
    errorBorder: OutlineInputBorder(
        borderRadius: BorderRadius.circular(100.0),
        borderSide: BorderSide(color: Colors.red, width: 2.0)),
    focusedErrorBorder: OutlineInputBorder(
        borderRadius: BorderRadius.circular(100.0),
        borderSide: BorderSide(color: Colors.red, width: 2.0)),
  );
}

BoxDecoration inputBoxDecorationShaddow() {
  return BoxDecoration(boxShadow: [
    BoxShadow(
      color: Colors.black.withOpacity(0.1),
      blurRadius: 20,
      offset: const Offset(0, 5),
    )
  ]);
}
